unit unitMainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, JvHidControllerClass,ComCtrls, ExtCtrls,Math,MMSystem;
const

  WIIREMOTE_VID = $057E;
  WIIREMOTE_PID = $0306;


type
  TMainForm = class(TForm)
    HIDCtl: TJvHidDeviceController;
    ListBox1: TListBox;
    EditX: TEdit;
    EditY: TEdit;
    EditZ: TEdit;
    imgGraph: TImage;
    Timer1: TTimer;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Timer2: TTimer;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    function HIDCtlEnumerate(HidDev: TJvHidDevice; const Index: Integer): Boolean;
    procedure HIDCtlDeviceChange(Sender: TObject);
    procedure ShowRead(HidDev: TJvHidDevice; ReportID: Byte; const Data: Pointer; Size: Word);
    procedure AddToList(Str: string);
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Timer2Timer(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    CurrentDevice: TJvHidDevice;
    BufX: array [0..1024] of integer;
    BufY: array [0..1024] of integer;
    BufZ: array [0..1024] of integer;

    TriggerHome:boolean;
    TriggerA:boolean;
    TriggerB:boolean;
    Trigger1:boolean;
    Trigger2:boolean;
    TriggerPlus:boolean;
    TriggerMinus:boolean;
    TriggerUp:boolean;
    TriggerDown:boolean;
    TriggerLeft:boolean;
    TriggerRight:boolean;

    prevTriggerBTime:integer;
    TriggerBEnabled:boolean;

    procedure AnalyzePacket(Str: string);
    procedure PushX(value: integer);
    procedure PushY(value: integer);
    procedure PushZ(value: integer);
    procedure DrawImage;
    procedure SensorStart;
    procedure SensorStop;
    procedure SendToDevice(Buf: array of Byte);
    procedure LEDCtrl(LEDState: integer);
    { Private 錾 }

  public
    { Public 錾 }
  end;

var
  MainForm: TMainForm;

implementation
uses unitTool;
{$R *.dfm}

function TMainForm.HIDCtlEnumerate(HidDev: TJvHidDevice; const Index: Integer): Boolean;
var
  oDev: TJvHidDevice;
begin
  if ((HidDev.Attributes.VendorID=WIIREMOTE_VID) and (HidDev.Attributes.ProductID=WIIREMOTE_PID)) then
  begin
    AddToList('WiiRemote Detected');
    begin
      if HIDCtl.CheckOutByID(oDev, WIIREMOTE_VID, WIIREMOTE_PID) then
      begin
        CurrentDevice := oDev;
        CurrentDevice.OnData := ShowRead;
      end;
    end;

    Result := False;
  end;
end;

procedure TMainForm.HIDCtlDeviceChange(Sender: TObject);
begin
HidCtl.Enumerate;
SensorStart;
end;


procedure TMainForm.ShowRead(HidDev: TJvHidDevice; ReportID: Byte; const Data: Pointer; Size: Word);
var
  I: Integer;
  Str: string;
begin
  Str := Format('R %.2x  ', [ReportID]);
  for I := 0 to Size - 1 do
    Str := Str + Format('%.2x ', [Cardinal(PChar(Data)[I])]);
  AnalyzePacket(Str);
  //AddToList(Str);
end;

procedure TMainForm.AddToList(Str: string);
begin
Listbox1.Items.Insert(0,'['+DateTimeToStr(Now)+'] '+Str);
end;

procedure TMainForm.PushX(value: integer);
var
  i:integer;
begin
  for i:=1023 downto 1 do BufX[i]:=BufX[i-1];
  BufX[0]:=value;
end;

procedure TMainForm.PushY(value: integer);
var
  i:integer;
begin
  for i:=1023 downto 1 do BufY[i]:=BufY[i-1];
  BufY[0]:=value;
end;

procedure TMainForm.PushZ(value: integer);
var
  i:integer;
begin
  for i:=1023 downto 1 do BufZ[i]:=BufZ[i-1];
  BufZ[0]:=value;
end;

procedure TMainForm.DrawImage();
var
  bmp:TBitMap;
  i:integer;
begin
  bmp:=TBitmap.Create;
  //bmp:= imgGraph.Picture.Bitmap;
  bmp.Width:=imgGraph.width;
  bmp.Height:=imgGraph.Height;

  bmp.Canvas.Pen.width:=2;

  bmp.Canvas.Pen.Color:=clblue;
  bmp.canvas.MoveTo(0,BufX[0]div 2);
  for i:= 0 to Min(bmp.Width,1023) do bmp.Canvas.LineTo(i,BufX[i] div 2);

  bmp.canvas.MoveTo(0,BufY[0]div 2);
  bmp.Canvas.Pen.Color:=clred;
  for i:= 0 to Min(bmp.Width,1023) do bmp.Canvas.LineTo(i,BufY[i]div 2);

  bmp.canvas.MoveTo(0,BufZ[0]div 2);
  bmp.Canvas.Pen.Color:=cllime;
  for i:= 0 to Min(bmp.Width,1023) do bmp.Canvas.LineTo(i,BufZ[i]div 2);

  //BitBlt(imgGraph.Canvas.Handle,0,0,bmp.width,bmp.height,bmp.Canvas.Handle,bmp.width,bmp.height,SRCCOPY);
  imgGraph.Picture.Bitmap:=bmp;
  bmp.Free;
end;

procedure TMainForm.AnalyzePacket(Str: string);
var
  data:string;
  button:integer;
  X,Y,Z:integer;
  ret:string;
  output:string;
  Point:TPoint;
begin
  data:=Mid(Str,7,2)+Mid(Str,10,2);
  X:=StrToInt('$'+Mid(Str,13,2));
  Y:=StrToInt('$'+Mid(Str,16,2));
  Z:=StrToInt('$'+Mid(Str,19,2));

  EditX.Text:=IntToStr(X);
  EditY.Text:=IntToStr(Y);
  EditZ.Text:=IntToStr(Z);
  
  PushX(X);
  PushY(Y);
  PushZ(Z);

  button:=StrToInt('$'+data);

  //{^
  if button and $0800 <> 0 then
  begin
    if TriggerUp=false then
    begin
      Keybd_event(VK_UP,0,0,0);
      Keybd_event(VK_UP,0,KEYEVENTF_KEYUP,0);
      ret:=ret+'up ';
    end;
    TriggerUp:=true;
  end
  else
  begin
    if TriggerUp then
    begin
    end;
    TriggerUp:=false;
  end;


  //{^
  if button and $0400 <> 0 then
  begin
    if TriggerDown=false then
    begin
      Keybd_event(VK_DOWN,0,0,0);
      Keybd_event(VK_DOWN,0,KEYEVENTF_KEYUP,0);
    ret:=ret+'down ';
    end;
    TriggerDown:=true;
  end
  else
  begin
    if TriggerDown then
    begin
    end;
    TriggerDown:=false;
  end;

  //E{^
  if button and $0200 <> 0 then
  begin
    if TriggerRight=false then
    begin
      Keybd_event(VK_RIGHT,0,0,0);
      Keybd_event(VK_RIGHT,0,KEYEVENTF_KEYUP,0);
      ret:=ret+'right ';
    end;
    TriggerRight:=true;
  end
  else
  begin
    if TriggerRight then
    begin
    end;
    TriggerRight:=false;
  end;

  //{^
  if button and $0100 <> 0 then
  begin
    if TriggerLeft=false then
    begin
      Keybd_event(VK_LEFT,0,0,0);
      Keybd_event(VK_LEFT,0,KEYEVENTF_KEYUP,0);
      ret:=ret+'left ';
    end;
    TriggerLeft:=true;
  end
  else
  begin
    if TriggerLeft then
    begin
    end;
    TriggerLeft:=false;
  end;

  //home{^
  if button and $0080 <> 0 then
  begin
    if TriggerHome=false then
    begin
       Mouse_Event(MOUSEEVENTF_RIGHTDOWN,0,0,0,0);
      ret:=ret+'home ';
    end;
    TriggerHome:=true;
  end
  else
  begin
    if TriggerHome then Mouse_Event(MOUSEEVENTF_RIGHTUP,0,0,0,0);
    TriggerHome:=false;
  end;

  //plus{^
  if button and $1000 <> 0 then
  begin
    if TriggerPlus=false then
    begin
      Keybd_event(VK_RETURN,0,0,0);
      Keybd_event(VK_RETURN,0,KEYEVENTF_KEYUP,0);
      ret:=ret+'plus ';
    end;
    TriggerPlus:=true;
  end
  else
  begin
    if TriggerPlus then
    begin
    end;
    TriggerPlus:=false;
  end;

  //minus{^
  if button and $0010 <> 0 then
  begin
    if TriggerMinus=false then
    begin
      Keybd_event(VK_BACK,0,0,0);
      Keybd_event(VK_BACK,0,KEYEVENTF_KEYUP,0);
      ret:=ret+'minus ';
    end;
    TriggerMinus:=true;
  end
  else
  begin
    if TriggerMinus then
    begin
    end;
    TriggerMinus:=false;
  end;

  //A{^
  if button and $0008 <> 0 then
  begin
    if TriggerA=false then
    begin
      Mouse_Event(MOUSEEVENTF_LEFTDOWN,0,0,0,0);
      ret:=ret+'A ';
    end;
    TriggerA:=true;
  end
  else
  begin
    if TriggerA=true then Mouse_Event(MOUSEEVENTF_LEFTUP,0,0,0,0);
    TriggerA:=false;
  end;

  //B{^
  if button and $0004 <> 0 then
  begin
    if TriggerB=false then
    begin
      TriggerBEnabled:=Timer2.Enabled;
      Timer2.Enabled:=true;
      LEDCtrl($10);

      prevTriggerBTime:=timeGetTime();
      ret:=ret+'B ';
    end;
    TriggerB:=true;
  end
  else
  begin
    if TriggerB=true then
    begin
      if (timegettime -prevTriggerBTime <300) then
      begin
        //tggled
        if TriggerBEnabled then
        begin
          Timer2.Enabled:=false;
          LEDCtrl($00);
          TriggerBEnabled:=false;
        end;
      end;
    end;
    TriggerB:=false;
  end;

  //1{^
  if button and $0002 <> 0 then
  begin
    if Trigger1=false then
    begin
    ret:=ret+'1 ';
    end;
    Trigger1:=true;
  end
  else
  begin
    if Trigger1 then
    begin
    end;
    Trigger1:=false;
  end;


  //2{^
  if button and $0001 <> 0 then
  begin
    if Trigger2=false then
    begin
      ret:=ret+'2 ';
    end;
    Trigger2:=true;
  end
  else
  begin
    if Trigger2 then
    begin
    end;
    Trigger2:=false;
  end;



  if data ='0000' then
  begin
    //output:='no button pressed';
  end;

  if ret<>'' then output:=ret+'pressed';

  if output<>'' then AddToList(output);

end;

procedure TMainForm.SensorStart();
var
  Buf: array [0..64] of Byte;
begin
  //݃f[^̗p
  Buf[0] := $12;  {ReportID}
  Buf[1] := $00;
  Buf[2] := $31;

  SendToDevice(Buf);
end;

procedure TMainForm.SensorStop();
var
  Buf: array [0..64] of Byte;
begin
  Buf[0] := $12;  {ReportID}
  Buf[1] := $00;
  Buf[2] := $30;
  SendToDevice(Buf);
end;

procedure TMainForm.LEDCtrl(LEDState:integer);
var
  Buf: array [0..64] of Byte;
begin
  Buf[0] := $11;  {ReportID}
  Buf[1] := LEDState;
  Buf[2] := $00;
  SendToDevice(Buf);
end;


procedure TMainForm.SendToDevice(Buf: array of Byte);
var
  numToWrite: Cardinal;
  numWritten: Cardinal;
  numErr: DWORD;
begin

  //foCX蓖ĂĂȂȂExit
  if not Assigned(CurrentDevice) then
  begin
    MainForm.Caption := 'Device not found';
    Exit;
  end;
  numToWrite := CurrentDevice.Caps.OutputReportByteLength;

  //foCXɑM
  CurrentDevice.OpenFile;
  if not CurrentDevice.WriteFile(Buf, numToWrite, numWritten) then
  begin
    //G[ꍇAbZ[W\
    numErr := GetLastError;
    MainForm.Caption := 'Error(' + IntToStr(numErr) + '):' + SysErrorMessage(numErr);
  end;
end;


procedure TMainForm.Timer1Timer(Sender: TObject);
begin
DrawImage();
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
MainForm.DoubleBuffered:=true;
end;

procedure TMainForm.Timer2Timer(Sender: TObject);
var
  Point:TPoint;
  dx,dy:integer;
begin
  GetCursorPos(Point);
  if abs(BufX[0]-127) <2 then dx:=0 else dx:=BufX[0]-127;
  if abs(BufY[0]-127) <2 then dy:=0 else dy:=BufY[0]-127;
  if TriggerB then
  begin
    dx:=dx div 5;
    dy:=dy div 5;
  end;

  SetCursorPos(Point.X+dx,Point.Y+dy);
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  LEDCtrl($00);
  SensorStop;
end;

end.
